@@ -80,7 +80,9 @@
 GConf conf = {
   .append_method = 1,
   .append_protocol = 1,
+  .chunk_size = 1024,
   .hl_header = 1,
+  .jobs = 1,
   .num_tests = 10,
 };
 
@@ -436,7 +438,7 @@ collapse_current_module (void) {
   return 0;
 }
 
-/* Display message a the bottom of the terminal dashboard that panel
+/* Display message at the bottom of the terminal dashboard that panel
  * is disabled */
 static void
 disabled_panel_msg (GModule module) {
@@ -566,7 +568,7 @@ expand_on_mouse_click (void) {
   return 1;
 }
 
-/* Scroll dowm expanded module to the last row */
+/* Scroll down expanded module to the last row */
 static void
 scroll_down_expanded_module (void) {
   int exp_size = get_num_expanded_data_rows ();
@@ -767,6 +769,7 @@ read_client (void *ptr_data) {
 /* Parse tailed lines */
 static void
 parse_tail_follow (GLog *glog, FILE *fp) {
+  GLogItem *logitem = NULL;
 #ifdef WITH_GETLINE
   char *buf = NULL;
 #else
@@ -780,7 +783,12 @@ parse_tail_follow (GLog *glog, FILE *fp) {
   while (fgets (buf, LINE_BUFFER, fp) != NULL) {
 #endif
     pthread_mutex_lock (&gdns_thread.mutex);
-    pre_process_log (glog, buf, 0);
+    if ((parse_line (glog, buf, 0, &logitem)) == 0 && logitem != NULL)
+      process_log (logitem);
+    if (logitem != NULL) {
+      free_glog (logitem);
+      logitem = NULL;
+    }
     pthread_mutex_unlock (&gdns_thread.mutex);
     glog->bytes += strlen (buf);
 #ifdef WITH_GETLINE
@@ -1015,6 +1023,14 @@ get_keys (Logs *logs) {
   int c, quit = 1;
   uint32_t offset = 0;
 
+  struct sigaction act, oldact;
+
+  /* Change the action for SIGINT to SIG_IGN and block Ctrl+c
+   * before entering the subdialog */
+  act.sa_handler = SIG_IGN;
+  sigemptyset (&act.sa_mask);
+  act.sa_flags = 0;
+
   while (quit) {
     if (conf.stop_processing)
       break;
@@ -1033,7 +1049,9 @@ get_keys (Logs *logs) {
     case KEY_F (1):
     case '?':
     case 'h':
+      sigaction (SIGINT, &act, &oldact);
       load_help_popup (main_win);
+      sigaction (SIGINT, &oldact, NULL);
       render_screens (offset);
       break;
     case 49:   /* 1 */
@@ -1204,20 +1222,29 @@ get_keys (Logs *logs) {
         render_screens (offset);
       break;
     case '/':
+      sigaction (SIGINT, &act, &oldact);
       if (render_search_dialog (search) == 0)
         render_screens (offset);
+      sigaction (SIGINT, &oldact, NULL);
       break;
     case 99:   /* c */
       if (conf.no_color)
         break;
+
+      sigaction (SIGINT, &act, &oldact);
       load_schemes_win (main_win);
+      sigaction (SIGINT, &oldact, NULL);
+
       free_dashboard (dash);
       allocate_data ();
       set_wbkgd (main_win, header_win);
       render_screens (offset);
       break;
     case 115:  /* s */
+      sigaction (SIGINT, &act, &oldact);
       render_sort_dialog ();
+      sigaction (SIGINT, &oldact, NULL);
+
       render_screens (offset);
       break;
     case 269:
@@ -1277,8 +1304,11 @@ standard_output (Logs *logs) {
   if (find_output_type (&json, "json", 1) == 0)
     output_json (holder, json);
   /* HTML */
-  if (find_output_type (&html, "html", 1) == 0 || conf.output_format_idx == 0)
+  if (find_output_type (&html, "html", 1) == 0 || conf.output_format_idx == 0) {
+    if (conf.real_time_html)
+      setup_ws_server (gwswriter, gwsreader);
     process_html (logs, html);
+  }
 
   free (csv);
   free (html);
@@ -1410,7 +1440,14 @@ parse_cmd_line (int argc, char **argv) {
 
 static void
 handle_signal_action (GO_UNUSED int sig_number) {
-  fprintf (stderr, "\nSIGINT caught!\n");
+  if (sig_number == SIGINT)
+    fprintf (stderr, "\nSIGINT caught!\n");
+  else if (sig_number == SIGTERM)
+    fprintf (stderr, "\nSIGTERM caught!\n");
+  else if (sig_number == SIGQUIT)
+    fprintf (stderr, "\nSIGQUIT caught!\n");
+  else
+    fprintf (stderr, "\nSignal %d caught!\n", sig_number);
   fprintf (stderr, "Closing GoAccess...\n");
 
   if (conf.output_stdout && conf.real_time_html)
@@ -1428,6 +1465,7 @@ setup_thread_signals (void) {
 
   sigaction (SIGINT, &act, NULL);
   sigaction (SIGTERM, &act, NULL);
+  sigaction (SIGQUIT, &act, NULL);
   signal (SIGPIPE, SIG_IGN);
 
   /* Restore old signal mask for the main thread */
@@ -1436,13 +1474,14 @@ setup_thread_signals (void) {
 
 static void
 block_thread_signals (void) {
-  /* Avoid threads catching SIGINT/SIGPIPE/SIGTERM and handle them in
+  /* Avoid threads catching SIGINT/SIGPIPE/SIGTERM/SIGQUIT and handle them in
    * main thread */
   sigset_t sigset;
   sigemptyset (&sigset);
   sigaddset (&sigset, SIGINT);
   sigaddset (&sigset, SIGPIPE);
   sigaddset (&sigset, SIGTERM);
+  sigaddset (&sigset, SIGQUIT);
   pthread_sigmask (SIG_BLOCK, &sigset, &oldset);
 }
 
@@ -1533,7 +1572,6 @@ spawn_ws (void) {
 
   if (conf.daemonize)
     daemonize ();
-  setup_ws_server (gwswriter, gwsreader);
 
   return 0;
 }
