@@ -28,10 +28,6 @@
  * SOFTWARE.
  */
 
-#include <ctype.h>
-#include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <stddef.h>
 
@@ -136,6 +132,8 @@ get_real_android (const char *droid) {
     return alloc_string ("Android 13");
   else if (strstr (droid, "12"))
     return alloc_string ("Android 12");
+  else if (strstr (droid, "12.1"))
+    return alloc_string ("Android 12.1");
   else if (strstr (droid, "11"))
     return alloc_string ("Android 11");
   else if (strstr (droid, "10"))
