@@ -37,7 +37,6 @@
 #include <string.h>
 #include <getopt.h>
 #include <errno.h>
-#include <unistd.h>
 
 #ifdef HAVE_LIBGEOIP
 #include <GeoIP.h>
@@ -51,11 +50,11 @@
 
 #include "xmalloc.h"
 
-static char short_options[] = "f:e:p:o:l:H:M:S:b:"
+static char short_options[] = "b:e:f:j:l:o:p:H:M:S:"
 #ifdef HAVE_LIBGEOIP
   "g"
 #endif
-  "acirmMhHqdsV";
+  "acdhimqrsV";
 
 /* *INDENT-OFF* */
 struct option long_opts[] = {
@@ -72,11 +71,12 @@ struct option long_opts[] = {
   {"hl-header"            , no_argument       , 0 , 'i' } ,
   {"http-method"          , required_argument , 0 , 'M' } ,
   {"http-protocol"        , required_argument , 0 , 'H' } ,
+  {"jobs"                 , required_argument , 0 , 'j' } ,
   {"log-file"             , required_argument , 0 , 'f' } ,
   {"log-size"             , required_argument , 0 , 'S' } ,
   {"no-query-string"      , no_argument       , 0 , 'q' } ,
   {"no-term-resolver"     , no_argument       , 0 , 'r' } ,
-  {"output-format"        , required_argument , 0 , 'o' } ,
+  {"output"               , required_argument , 0 , 'o' } ,
   {"storage"              , no_argument       , 0 , 's' } ,
   {"version"              , no_argument       , 0 , 'V' } ,
   {"with-mouse"           , no_argument       , 0 , 'm' } ,
@@ -91,6 +91,7 @@ struct option long_opts[] = {
   {"color"                , required_argument , 0 , 0  }  ,
   {"color-scheme"         , required_argument , 0 , 0  }  ,
   {"crawlers-only"        , no_argument       , 0 , 0  }  ,
+  {"chunk-size"           , required_argument , 0 , 0  }  ,
   {"daemonize"            , no_argument       , 0 , 0  }  ,
   {"datetime-format"      , required_argument , 0 , 0  }  ,
   {"date-format"          , required_argument , 0 , 0  }  ,
@@ -133,7 +134,7 @@ struct option long_opts[] = {
   {"no-tab-scroll"        , no_argument       , 0 , 0  }  ,
   {"num-tests"            , required_argument , 0 , 0  }  ,
   {"origin"               , required_argument , 0 , 0  }  ,
-  {"output"               , required_argument , 0 , 0  }  ,
+  {"output-format"        , required_argument , 0 , 0  }  ,
   {"persist"              , no_argument       , 0 , 0  }  ,
   {"pid-file"             , required_argument , 0 , 0  }  ,
   {"port"                 , required_argument , 0 , 0  }  ,
@@ -262,9 +263,12 @@ cmd_help (void)
   "  -d --with-output-resolver       - Enable IP resolver on HTML|JSON output.\n"
   "  -e --exclude-ip=<IP>            - Exclude one or multiple IPv4/6. Allows IP\n"
   "                                    ranges. e.g., 192.168.0.1-192.168.0.10\n"
+  "  -j --jobs=<1-6>                 - Thread count for parsing log. Defaults to 1.\n"
+  "                                    The use of 2-4 threads is recommended.\n"
   "  -H --http-protocol=<yes|no>     - Set/unset HTTP request protocol if found.\n"
   "  -M --http-method=<yes|no>       - Set/unset HTTP request method if found.\n"
-  "  -o --output=file.html|json|csv  - Output either an HTML, JSON or a CSV file.\n"
+  "  -o --output=<format|filename>   - Output to stdout or the specified file.\n"
+  "                                    e.g., -o csv, -o out.json, --output=report.html\n"
   "  -q --no-query-string            - Strip request's query string. This can\n"
   "                                    decrease memory consumption.\n"
   "  -r --no-term-resolver           - Disable IP resolver on terminal output.\n"
@@ -276,6 +280,8 @@ cmd_help (void)
   "                                    report.\n"
   "  --anonymize-level=<1|2|3>       - Anonymization levels: 1 => default, 2 =>\n"
   "                                    strong, 3 => pedantic.\n"
+  "  --chunk-size=<256-32768>        - Number of lines processed in each data chunk\n"
+  "                                    for parallel execution. Default is 1024.\n"
   "  --crawlers-only                 - Parse and display only crawlers.\n"
   "  --date-spec=<date|hr|min>       - Date specificity. Possible values: `date`\n"
   "                                    (default), `hr` or `min`.\n"
@@ -553,8 +559,8 @@ parse_long_opt (const char *name, const char *oarg) {
   }
 
   /* output file */
-  if (!strcmp ("output", name))
-    set_array_opt (oarg, conf.output_formats, &conf.output_format_idx, MAX_OUTFORMATS);
+  if (!strcmp ("output-format", name))
+    FATAL ("The option --output-format is deprecated, please use --output instead.");
 
   /* PARSE OPTIONS
    * ========================= */
@@ -578,6 +584,16 @@ parse_long_opt (const char *name, const char *oarg) {
   if (!strcmp ("all-static-files", name))
     conf.all_static_files = 1;
 
+  /* chunk size */
+  if (!strcmp ("chunk-size", name)) {
+    /* Recommended chunk size is 256 - 32768, hard limit is 32 - 1048576. */
+    conf.chunk_size = atoi (oarg);
+    if (conf.chunk_size < 32)
+      FATAL ("The hard lower limit of --chunk-size is 32.");
+    if (conf.chunk_size > 1048576)
+      FATAL ("The hard limit of --chunk-size is 1048576.");
+  }
+
   /* crawlers only */
   if (!strcmp ("crawlers-only", name))
     conf.crawlers_only = 1;
@@ -627,7 +643,8 @@ parse_long_opt (const char *name, const char *oarg) {
 
   /* ignore status code */
   if (!strcmp ("ignore-status", name))
-    set_array_opt (oarg, conf.ignore_status, &conf.ignore_status_idx, MAX_IGNORE_STATUS);
+    if (conf.ignore_status_idx < MAX_IGNORE_STATUS)
+      conf.ignore_status[conf.ignore_status_idx++] = atoi (oarg);
 
   /* ignore static requests */
   if (!strcmp ("ignore-statics", name)) {
@@ -804,6 +821,12 @@ read_option_args (int argc, char **argv) {
     case 'i':
       conf.hl_header = 1;
       break;
+    case 'j':
+      /* Recommended 4 threads, soft limit is 6, hard limit is 12. */
+      conf.jobs = atoi (optarg);
+      if (conf.jobs > 12)
+        FATAL ("The hard limit of --jobs is 12.");
+      break;
     case 'q':
       conf.ignore_qstr = 1;
       break;
