@@ -46,18 +46,16 @@
 #include <config.h>
 #endif
 
-#include <arpa/inet.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <strings.h>
 #include <sys/stat.h>
-#include <sys/types.h>
 #include <unistd.h>
-#include <sys/stat.h>
 #include <fcntl.h>
 #include <inttypes.h>
+#include <pthread.h>
 #include <libgen.h>
 
 #include "gkhash.h"
@@ -68,7 +66,6 @@
 #include "error.h"
 #include "goaccess.h"
 #include "gstorage.h"
-#include "pdjson.h"
 #include "util.h"
 #include "websocket.h"
 #include "xmalloc.h"
@@ -256,10 +253,8 @@ free_logs (Logs *logs) {
  * On success, the new GLogItem instance is returned. */
 GLogItem *
 init_log_item (GLog *glog) {
-  time_t now = time (0);
   GLogItem *logitem;
-  glog->items = xmalloc (sizeof (GLogItem));
-  logitem = glog->items;
+  logitem = xmalloc (sizeof (GLogItem));
   memset (logitem, 0, sizeof *logitem);
 
   logitem->agent = NULL;
@@ -282,7 +277,7 @@ init_log_item (GLog *glog) {
   logitem->req = NULL;
   logitem->resp_size = 0LL;
   logitem->serve_time = 0;
-  logitem->status = NULL;
+  logitem->status = 0;
   logitem->time = NULL;
   logitem->uniq_key = NULL;
   logitem->vhost = NULL;
@@ -297,13 +292,13 @@ init_log_item (GLog *glog) {
 
   memset (logitem->site, 0, sizeof (logitem->site));
   memset (logitem->agent_hex, 0, sizeof (logitem->agent_hex));
-  localtime_r (&now, &logitem->dt);
+  logitem->dt = glog->start_time;
 
   return logitem;
 }
 
 /* Free all members of a GLogItem */
-static void
+void
 free_glog (GLogItem *logitem) {
   if (logitem->agent != NULL)
     free (logitem->agent);
@@ -341,8 +336,6 @@ free_glog (GLogItem *logitem) {
     free (logitem->req_key);
   if (logitem->req != NULL)
     free (logitem->req);
-  if (logitem->status != NULL)
-    free (logitem->status);
   if (logitem->time != NULL)
     free (logitem->time);
   if (logitem->uniq_key != NULL)
@@ -484,7 +477,7 @@ extract_referer_site (const char *referer, char *host) {
   if ((len = strlen (begin)) == 0)
     goto clean;
 
-  if ((end = strchr (begin, '/')) != NULL)
+  if ((end = strpbrk (begin, "/?")) != NULL)
     len = end - begin;
 
   if (len == 0)
@@ -553,14 +546,6 @@ extract_method (const char *token) {
   return NULL;
 }
 
-/* Determine if time-served data was stored on-disk. */
-static void
-contains_usecs (void) {
-  if (conf.serve_usecs)
-    return;
-  conf.serve_usecs = 1; /* flag */
-}
-
 static int
 is_cache_hit (const char *tkn) {
   if (strcasecmp ("MISS", tkn) == 0)
@@ -724,7 +709,7 @@ get_delim (char *dest, const char *p) {
  *
  * On success, the malloc'd token is returned. */
 static char *
-parsed_string (const char *pch, char **str, int move_ptr) {
+parsed_string (const char *pch, const char **str, int move_ptr) {
   char *p;
   size_t len = (pch - *str + 1);
 
@@ -742,9 +727,9 @@ parsed_string (const char *pch, char **str, int move_ptr) {
  * On error, or unable to parse it, NULL is returned.
  * On success, the malloc'd token is returned. */
 static char *
-parse_string (char **str, const char *delims, int cnt) {
+parse_string (const char **str, const char *delims, int cnt) {
   int idx = 0;
-  char *pch = *str, *p = NULL;
+  const char *pch = *str, *p = NULL;
   char end;
 
   if ((*delims != 0x0) && (p = strpbrk (*str, delims)) == NULL)
@@ -767,15 +752,15 @@ parse_string (char **str, const char *delims, int cnt) {
 }
 
 char *
-extract_by_delim (char **str, const char *end) {
+extract_by_delim (const char **str, const char *end) {
   return parse_string (&(*str), end, 1);
 }
 
 /* Move forward through the log string until a non-space (!isspace)
  * char is found. */
 static void
-find_alpha (char **str) {
-  char *s = *str;
+find_alpha (const char **str) {
+  const char *s = *str;
   while (*s) {
     if (isspace (*s))
       s++;
@@ -788,9 +773,9 @@ find_alpha (char **str) {
 /* Move forward through the log string until a non-space (!isspace)
  * char is found and returns the count. */
 static int
-find_alpha_count (char *str) {
+find_alpha_count (const char *str) {
   int cnt = 0;
-  char *s = str;
+  const char *s = str;
   while (*s) {
     if (isspace (*s))
       s++, cnt++;
@@ -905,22 +890,20 @@ set_agent_hash (GLogItem *logitem) {
  * On error, or unable to parse it, 1 is returned.
  * On success, the malloc'd token is assigned to a GLogItem member. */
 static int
-parse_specifier (GLogItem *logitem, char **str, const char *p, const char *end) {
+parse_specifier (GLogItem *logitem, const char **str, const char *p, const char *end) {
   struct tm tm;
-  time_t now = time (0);
   const char *dfmt = conf.date_format;
   const char *tfmt = conf.time_format;
 
   char *pch, *sEnd, *bEnd, *tkn = NULL;
   double serve_secs = 0.0;
   uint64_t bandw = 0, serve_time = 0;
-  long status = 0L;
   int dspc = 0, fmtspcs = 0;
 
   errno = 0;
   memset (&tm, 0, sizeof (tm));
   tm.tm_isdst = -1;
-  localtime_r (&now, &tm);
+  tm = logitem->dt;
 
   switch (*p) {
     /* date */
@@ -1123,19 +1106,14 @@ parse_specifier (GLogItem *logitem, char **str, const char *p, const char *end)
     if (!(tkn = parse_string (&(*str), end, 1)))
       return spec_err (logitem, ERR_SPEC_TOKN_NUL, *p, NULL);
 
-    /* do not validate HTTP status code */
-    if (conf.no_strict_status) {
-      logitem->status = tkn;
-      break;
-    }
-
-    status = strtol (tkn, &sEnd, 10);
-    if (tkn == sEnd || *sEnd != '\0' || errno == ERANGE || status < 100 || status > 599) {
+    logitem->status = strtol (tkn, &sEnd, 10);
+    if (tkn == sEnd || *sEnd != '\0' || errno == ERANGE ||
+        (!conf.no_strict_status && (logitem->status < 100 || logitem->status > 599))) {
       spec_err (logitem, ERR_SPEC_TOKN_INV, *p, tkn);
       free (tkn);
       return 1;
     }
-    logitem->status = tkn;
+    free (tkn);
     break;
     /* size of response in bytes - excluding HTTP headers */
   case 'b':
@@ -1148,7 +1126,7 @@ parse_specifier (GLogItem *logitem, char **str, const char *p, const char *end)
     if (tkn == bEnd || *bEnd != '\0' || errno == ERANGE)
       bandw = 0;
     logitem->resp_size = bandw;
-    conf.bandwidth = 1;
+    __sync_bool_compare_and_swap (&conf.bandwidth, 0, 1);       /* set flag */
     free (tkn);
     break;
     /* referrer */
@@ -1216,7 +1194,8 @@ parse_specifier (GLogItem *logitem, char **str, const char *p, const char *end)
     /* convert it to microseconds */
     logitem->serve_time = (serve_secs > 0) ? serve_secs * MILS : 0;
 
-    contains_usecs ();  /* set flag */
+    /* Determine if time-served data was stored on-disk. */
+    __sync_bool_compare_and_swap (&conf.serve_usecs, 0, 1);     /* set flag */
     free (tkn);
     break;
     /* time taken to serve the request, in seconds with a milliseconds
@@ -1238,7 +1217,8 @@ parse_specifier (GLogItem *logitem, char **str, const char *p, const char *end)
     /* convert it to microseconds */
     logitem->serve_time = (serve_secs > 0) ? serve_secs * SECS : 0;
 
-    contains_usecs ();  /* set flag */
+    /* Determine if time-served data was stored on-disk. */
+    __sync_bool_compare_and_swap (&conf.serve_usecs, 0, 1);     /* set flag */
     free (tkn);
     break;
     /* time taken to serve the request, in microseconds */
@@ -1254,7 +1234,8 @@ parse_specifier (GLogItem *logitem, char **str, const char *p, const char *end)
       serve_time = 0;
     logitem->serve_time = serve_time;
 
-    contains_usecs ();  /* set flag */
+    /* Determine if time-served data was stored on-disk. */
+    __sync_bool_compare_and_swap (&conf.serve_usecs, 0, 1);     /* set flag */
     free (tkn);
     break;
     /* time taken to serve the request, in nanoseconds */
@@ -1272,7 +1253,8 @@ parse_specifier (GLogItem *logitem, char **str, const char *p, const char *end)
     /* convert it to microseconds */
     logitem->serve_time = (serve_time > 0) ? serve_time / MILS : 0;
 
-    contains_usecs ();  /* set flag */
+    /* Determine if time-served data was stored on-disk. */
+    __sync_bool_compare_and_swap (&conf.serve_usecs, 0, 1);     /* set flag */
     free (tkn);
     break;
     /* UMS: Krypto (TLS) "ECDHE-RSA-AES128-GCM-SHA256" */
@@ -1339,8 +1321,9 @@ parse_specifier (GLogItem *logitem, char **str, const char *p, const char *end)
  * If no unable to find both curly braces (boundaries), NULL is returned.
  * On success, the malloc'd reject set is returned. */
 static char *
-extract_braces (char **p) {
-  char *b1 = NULL, *b2 = NULL, *ret = NULL, *s = *p;
+extract_braces (const char **p) {
+  const char *b1 = NULL, *b2 = NULL, *s = *p;
+  char *ret = NULL;
   int esc = 0;
   ptrdiff_t len = 0;
 
@@ -1378,8 +1361,8 @@ extract_braces (char **p) {
  * On success, the malloc'd token is assigned to a GLogItem->host and
  * 0 is returned. */
 static int
-set_xff_host (GLogItem *logitem, char *str, char *skips, int out) {
-  char *ptr = NULL, *tkn = NULL;
+set_xff_host (GLogItem *logitem, const char *str, const char *skips, int out) {
+  const char *ptr = NULL, *tkn = NULL;
   int invalid_ip = 1, len = 0, type_ip = TYPE_IPINV;
   int idx = 0, skips_len = 0;
 
@@ -1403,14 +1386,14 @@ set_xff_host (GLogItem *logitem, char *str, char *skips, int out) {
     invalid_ip = invalid_ipaddr (tkn, &type_ip);
     /* done, already have IP and current token is not a host */
     if (logitem->host && invalid_ip) {
-      free (tkn);
+      free ((void *) tkn);
       break;
     }
     if (!logitem->host && !invalid_ip) {
       logitem->host = xstrdup (tkn);
       logitem->type_ip = type_ip;
     }
-    free (tkn);
+    free ((void *) tkn);
     idx = 0;
 
     /* found the client IP, break then */
@@ -1429,7 +1412,7 @@ set_xff_host (GLogItem *logitem, char *str, char *skips, int out) {
  * If no IP is found, 1 is returned.
  * On success, the malloc'd token is assigned to a GLogItem->host and 0 is returned. */
 static int
-find_xff_host (GLogItem *logitem, char **str, char **p) {
+find_xff_host (GLogItem *logitem, const char **str, const char **p) {
   char *skips = NULL, *extract = NULL;
   char pch[2] = { 0 };
   int res = 0;
@@ -1464,7 +1447,7 @@ clean:
  * On success, the malloc'd token is assigned to a GLogItem member and
  * 0 is returned. */
 static int
-special_specifier (GLogItem *logitem, char **str, char **p) {
+special_specifier (GLogItem *logitem, const char **str, const char **p) {
   switch (**p) {
     /* XFF remote hostname (IP only) */
   case 'h':
@@ -1482,9 +1465,9 @@ special_specifier (GLogItem *logitem, char **str, char **p) {
  * On success, the malloc'd token is assigned to a GLogItem member and
  * 0 is returned. */
 static int
-parse_format (GLogItem *logitem, char *str, char *lfmt) {
+parse_format (GLogItem *logitem, const char *str, const char *lfmt) {
   char end[2 + 1] = { 0 };
-  char *p = NULL, *last = NULL;
+  const char *p = NULL, *last = NULL;
   int perc = 0, tilde = 0, ret = 0;
 
   if (str == NULL || *str == '\0')
@@ -1637,12 +1620,14 @@ is_static (const char *req) {
  * If the status code is not within the ignore-array, 0 is returned.
  * If the status code is within the ignore-array, 1 is returned. */
 static int
-ignore_status_code (const char *status) {
+ignore_status_code (int status) {
   if (!status || conf.ignore_status_idx == 0)
     return 0;
 
-  if (str_inarray (status, conf.ignore_status, conf.ignore_status_idx) != -1)
-    return 1;
+  for (int i = 0; i < conf.ignore_status_idx; i++)
+    if (status == conf.ignore_status[i])
+      return 1;
+
   return 0;
 }
 
@@ -1663,13 +1648,11 @@ ignore_static (const char *req) {
  * If the request is a 404, 1 is returned. */
 static int
 is_404 (GLogItem *logitem) {
-  if (!logitem->status || *logitem->status == '\0')
-    return 0;
   /* is this a 404? */
-  if (!memcmp (logitem->status, "404", 3))
+  if (logitem->status == 404)
     return 1;
   /* treat 444 as 404? */
-  else if (!memcmp (logitem->status, "444", 3) && conf.code444_as_404)
+  else if (logitem->status == 444 && conf.code444_as_404)
     return 1;
   return 0;
 }
@@ -1793,8 +1776,8 @@ should_restore_from_disk (GLog *glog) {
   else if (glog->props.size < lp.size && glog->lp.ts == lp.ts)
     return 0;
 
-  /* Everything else we ignore it. For instance, we if current log size is
-   * greater than the one last parsed, if the timestamp are equal, we ignore the
+  /* Everything else we ignore it. For instance, if current log size is
+   * greater than the one last parsed, or the timestamp are equal, we ignore the
    * request.
    *
    * **NOTE* We try to play safe here as we would rather miss a few lines
@@ -1808,7 +1791,7 @@ process_invalid (GLog *glog, GLogItem *logitem, const char *line) {
 
   /* if not restoring from disk, then count entry as proceeded and invalid */
   if (!conf.restore) {
-    count_process_and_invalid (glog, line);
+    count_process_and_invalid (glog, logitem, line);
     return;
   }
 
@@ -1819,13 +1802,13 @@ process_invalid (GLog *glog, GLogItem *logitem, const char *line) {
   if (glog->props.inode && is_likely_same_log (glog, &lp)) {
     /* only count invalids if we're past the last parsed line */
     if (glog->props.size > lp.size && glog->read >= lp.line)
-      count_process_and_invalid (glog, line);
+      count_process_and_invalid (glog, logitem, line);
     return;
   }
 
   /* no timestamp to compare against, just count the invalid then */
   if (!logitem->numdate) {
-    count_process_and_invalid (glog, line);
+    count_process_and_invalid (glog, logitem, line);
     return;
   }
 
@@ -1837,7 +1820,7 @@ process_invalid (GLog *glog, GLogItem *logitem, const char *line) {
    * then we simply don't count the entry as proceed & invalid to attempt over
    * counting restored data */
   if (should_restore_from_disk (glog) == 0)
-    count_process_and_invalid (glog, line);
+    count_process_and_invalid (glog, logitem, line);
 }
 
 static int
@@ -1865,16 +1848,44 @@ parse_json_format (GLogItem *logitem, char *str) {
   return parse_json_string (logitem, str, parse_json_specifier);
 }
 
+/* Atomically updates glog->lp.ts with the maximum timestamp value from
+ * logitem->dt.
+ *
+ * On error (if mktime fails), returns -1.
+ * On success, returns the updated timestamp value, which is also stored in
+ * glog->lp.ts.
+ */
+static int
+atomic_lpts_update (GLog *glog, GLogItem *logitem) {
+  int64_t oldts = 0, newts = 0;
+  /* atomic update loop */
+  newts = mktime (&logitem->dt);        // Get timestamp from logitem->dt
+  while (!__sync_bool_compare_and_swap (&glog->lp.ts, oldts, newts)) {
+    oldts = glog->lp.ts;        /* Reread glog->lp.ts if CAS failed */
+    if (oldts >= newts) {
+      break;    /* No need to update if oldts is already greater */
+    }
+  }
+
+  return newts;
+}
+
+static int
+cleanup_logitem (int ret, GLogItem *logitem) {
+  free_glog (logitem);
+  return ret;
+}
+
 /* Process a line from the log and store it accordingly taking into
  * account multiple parsing options prior to setting data into the
  * corresponding data structure.
  *
- * On success, 0 is returned */
+ * On error, logitem->errstr will contains the error message. */
 int
-pre_process_log (GLog *glog, char *line, int dry_run) {
-  GLogItem *logitem;
-  int ret = 0;
+parse_line (GLog *glog, char *line, int dry_run, GLogItem **logitem_out) {
   char *fmt = conf.log_format;
+  int ret = 0;
+  GLogItem *logitem = NULL;
 
   /* soft ignore these lines */
   if (valid_line (line))
@@ -1888,36 +1899,44 @@ pre_process_log (GLog *glog, char *line, int dry_run) {
   else
     ret = parse_format (logitem, line, fmt);
 
+  /* invalid log line (format issue) */
+  if (ret) {
+    process_invalid (glog, logitem, line);
+    return cleanup_logitem (ret, logitem);
+  }
+
   if (!glog->piping && conf.fname_as_vhost && glog->fname_as_vhost)
     logitem->vhost = xstrdup (glog->fname_as_vhost);
 
+  /* valid format but missing fields */
   if (ret || (ret = verify_missing_fields (logitem))) {
     process_invalid (glog, logitem, line);
-    goto cleanup;
+    return cleanup_logitem (ret, logitem);
   }
 
-  if ((glog->lp.ts = mktime (&logitem->dt)) == -1)
-    goto cleanup;
+  /* From here on, valid format but possible ignoring of lines */
+  if (atomic_lpts_update (glog, logitem) == -1)
+    return cleanup_logitem (ret, logitem);
 
   if (should_restore_from_disk (glog))
-    goto cleanup;
+    return cleanup_logitem (ret, logitem);
 
   count_process (glog);
 
+  /* testing log only */
+  if (dry_run)
+    return cleanup_logitem (ret, logitem);
+
   /* agent will be null in cases where %u is not specified */
   if (logitem->agent == NULL) {
     logitem->agent = alloc_string ("-");
     set_agent_hash (logitem);
   }
 
-  /* testing log only */
-  if (dry_run)
-    goto cleanup;
-
   logitem->ignorelevel = ignore_line (logitem);
   /* ignore line */
   if (logitem->ignorelevel == IGNORE_LEVEL_PANEL)
-    goto cleanup;
+    return cleanup_logitem (ret, logitem);
 
   if (is_404 (logitem))
     logitem->is_404 = 1;
@@ -1925,40 +1944,61 @@ pre_process_log (GLog *glog, char *line, int dry_run) {
     logitem->is_static = 1;
 
   logitem->uniq_key = get_uniq_visitor_key (logitem);
-
-  process_log (logitem);
-
-cleanup:
-  free_glog (logitem);
+  *logitem_out = logitem;
 
   return ret;
 }
 
-/* Entry point to process the given live from the log.
+/* Entry point to process the given line from the log.
  *
- * On error, 1 is returned.
- * On success or soft ignores, 0 is returned. */
-static int
-read_line (GLog *glog, char *line, int *test, int *cnt, int dry_run) {
+ * On error, NULL is returned.
+ * On success or soft ignores, GLogItem is returned. */
+static GLogItem *
+read_line (GLog *glog, char *line, int *test, uint32_t *cnt, int dry_run) {
+  GLogItem *logitem = NULL;
   int ret = 0;
 
-  /* start processing log line */
-  if ((ret = pre_process_log (glog, line, dry_run)) == 0 && *test)
+  /* Begin processing the log line - in case of an invalid log format, flip
+   * the test only if there's at least one valid record discovered during the log
+   * format test. This condition applies solely when reading a log from the
+   * beginning, not when tailing an ongoing log. */
+  if ((ret = parse_line (glog, line, dry_run, &logitem)) == 0)
     *test = 0;
 
-  /* soft ignores */
+  /* soft ignore these lines from parse_line */
   if (ret == -1)
-    return 0;
+    return NULL;
 
   /* reached num of lines to test and no valid records were found, log
    * format is likely not matching */
-  if (conf.num_tests && ++(*cnt) == (int) conf.num_tests && *test) {
+  if (conf.num_tests && ++(*cnt) >= conf.num_tests && *test) {
     uncount_processed (glog);
     uncount_invalid (glog);
-    return 1;
+    return NULL;
   }
 
-  return 0;
+  return logitem;
+}
+
+/* Parse chunk of lines to logitems */
+static void *
+read_lines_thread (void *arg) {
+  GJob *job = (GJob *) arg;
+
+  for (int i = 0; i < job->p; i++) {
+    /* ensure we don't process more than we should when testing for log format,
+     * else free chunk and stop processing threads */
+    if (!job->test || (job->test && job->cnt < conf.num_tests))
+      job->logitems[i] =
+        read_line (job->glog, job->lines[i], &job->test, &job->cnt, job->dry_run);
+    else
+      conf.stop_processing = 1;
+
+#ifdef WITH_GETLINE
+    free (job->lines[i]);
+#endif
+  }
+  return (void *) 0;
 }
 
 /* A replacement for GNU getline() to dynamically expand fgets buffer.
@@ -2005,84 +2045,187 @@ fgetline (FILE *fp) {
   return NULL;
 }
 
-/* Iterate over the log and read line by line (use GNU get_line to parse the
- * whole line).
- *
- * On error, 1 is returned.
- * On success, 0 is returned. */
-#ifdef WITH_GETLINE
-static int
-read_lines (FILE *fp, GLog *glog, int dry_run) {
-  char *line = NULL;
-  int ret = 0, cnt = 0, test = conf.num_tests > 0 ? 1 : 0;
-
-  glog->bytes = 0;
-  while ((line = fgetline (fp)) != NULL) {
-    /* handle SIGINT */
-    if (conf.stop_processing)
-      goto out;
-    if ((ret = read_line (glog, line, &test, &cnt, dry_run)))
-      goto out;
-    if (dry_run && NUM_TESTS == cnt)
-      goto out;
-    glog->bytes += strlen (line);
-    free (line);
-    glog->read++;
+void *
+process_lines_thread (void *arg) {
+  GJob *job = (GJob *) arg;
+  for (int i = 0; i < job->p; i++) {
+    if (job->logitems[i] != NULL && !job->dry_run && job->logitems[i]->errstr == NULL) {
+      process_log (job->logitems[i]);
+      free_glog (job->logitems[i]);
+    }
   }
-
-  /* if no data was available to read from (probably from a pipe) and
-   * still in test mode, we simply return until data becomes available */
-  if (!line && (errno == EAGAIN || errno == EWOULDBLOCK) && test)
-    return 0;
-
-  return (line && test) || ret || (!line && test && glog->processed);
-
-out:
-  free (line);
-  /* fails if
-     - we're still reading the log but the test flag was still set
-     - ret flag is not 0, read_line failed
-     - reached the end of file, test flag was still set and we processed lines */
-  return test || ret || (test && glog->processed);
+  return (void *) 0;
 }
-#endif
 
-/* Iterate over the log and read line by line (uses a buffer of fixed size).
- *
- * On error, 1 is returned.
- * On success, 0 is returned. */
+/* Initialize jobs */
+static void
+init_jobs (GJob jobs[2][conf.jobs], GLog *glog, int dry_run, int test) {
+  int b = 0, k = 0;
+
+  for (b = 0; b < 2; b++) {
+    for (k = 0; k < conf.jobs; k++) {
+      jobs[b][k].p = 0;
+      jobs[b][k].cnt = 0;
+      jobs[b][k].glog = glog;
+      jobs[b][k].test = test;
+      jobs[b][k].dry_run = dry_run;
+      jobs[b][k].running = 0;
+      jobs[b][k].logitems = xcalloc (conf.chunk_size, sizeof (GLogItem));
+      jobs[b][k].lines = xcalloc (conf.chunk_size, sizeof (char *));
 #ifndef WITH_GETLINE
+      for (int i = 0; i < conf.chunk_size; i++)
+        jobs[b][k].lines[i] = xcalloc (LINE_BUFFER, sizeof (char));
+#endif
+    }
+  }
+}
+
+/* Read lines from file */
+static void
+read_lines_from_file (FILE *fp, GLog *glog, GJob jobs[2][conf.jobs], int b, char **s) {
+  int k = 0;
+
+  for (k = 1; k < conf.jobs || (conf.jobs == 1 && k == 1); k++) {
+#ifdef WITH_GETLINE
+    while ((*s = fgetline (fp)) != NULL) {
+      jobs[b][k].lines[jobs[b][k].p] = *s;
+#else
+    while ((*s = fgets (jobs[b][k].lines[jobs[b][k].p], LINE_BUFFER, fp)) != NULL) {
+#endif
+      glog->bytes += strlen (jobs[b][k].lines[jobs[b][k].p]);
+      glog->read++;
+
+      if (++(jobs[b][k].p) >= conf.chunk_size)
+        break;  // goto next chunk
+    }
+  }
+}
+
+/* Processes lines using threads from the GJob array, updating counters. */
+static void
+process_lines (GJob jobs[2][conf.jobs], uint32_t *cnt, int *test, int b) {
+  int k = 0;
+
+  for (k = 1; k < conf.jobs || (conf.jobs == 1 && k == 1); k++) {
+    process_lines_thread (&jobs[b][k]);
+    *cnt += jobs[b][k].cnt;
+    jobs[b][k].cnt = 0;
+    *test &= jobs[b][k].test;
+    jobs[b][k].p = 0;
+  }
+}
+
+/* Frees memory for lines and logitems in each job of the GJob array. */
+static void
+free_jobs (GJob jobs[2][conf.jobs]) {
+  int b = 0, k = 0;
+
+  for (b = 0; b < 2; b++) {
+    for (k = 0; k < conf.jobs; k++) {
+#ifndef WITH_GETLINE
+      for (int i = 0; i < conf.chunk_size; i++)
+        free (jobs[b][k].lines[i]);
+#endif
+      free (jobs[b][k].logitems);
+      free (jobs[b][k].lines);
+    }
+  }
+}
+
+/* Reads lines from the given file pointer `fp` and processes them using
+ * parallel threads.
+ *
+ * On error or when interrupted by a signal (SIGINT), the function returns 0.
+ * On success, it returns 1 if the number of processed lines is greater than or
+ * equal to the configured number of tests (NUM_TESTS), otherwise 0.
+ */
 static int
 read_lines (FILE *fp, GLog *glog, int dry_run) {
+  int b = 0, k = 0, test = conf.num_tests > 0 ? 1 : 0;
+  uint32_t cnt = 0;
+  void *status = NULL;
   char *s = NULL;
-  char line[LINE_BUFFER] = { 0 };
-  int ret = 0, cnt = 0, test = conf.num_tests > 0 ? 1 : 0;
+  GJob jobs[2][conf.jobs];
+  pthread_t threads[conf.jobs];
 
   glog->bytes = 0;
-  while ((s = fgets (line, LINE_BUFFER, fp)) != NULL) {
+
+  init_jobs (jobs, glog, dry_run, test);
+
+  b = 0;
+  while (1) {   /* b = 0 or 1 */
+    read_lines_from_file (fp, glog, jobs, b, &s);
+
+    if (conf.jobs == 1) {
+      read_lines_thread (&jobs[b][1]);
+    } else {
+      for (k = 1; k < conf.jobs; k++) {
+        jobs[b][k].running = 1;
+        pthread_create (&threads[k], NULL, read_lines_thread, (void *) &jobs[b][k]);
+      }
+    }
+
+    /* flip from block A/B to B/A */
+    if (conf.jobs > 1)
+      b = b ^ 1;
+
+    process_lines (jobs, &cnt, &test, b);
+
+    /* flip from block B/A to A/B */
+    if (conf.jobs > 1)
+      b = b ^ 1;
+
+    for (k = 1; k < conf.jobs; k++) {
+      if (jobs[b][k].running) {
+        pthread_join (threads[k], &status);
+        jobs[b][k].running = 0;
+      }
+    }
+
+    if (dry_run && cnt >= NUM_TESTS)
+      break;
+
     /* handle SIGINT */
     if (conf.stop_processing)
       break;
-    if ((ret = read_line (glog, line, &test, &cnt, dry_run)))
+
+    /* check for EOF */
+    if (s == NULL)
       break;
-    if (dry_run && NUM_TESTS == cnt)
-      break;
-    glog->bytes += strlen (line);
-    glog->read++;
+
+    /* flip from block A/B to B/A */
+    if (conf.jobs > 1)
+      b = b ^ 1;
+  }     // while (1)
+
+  /* After eof, process last data */
+  for (b = 0; b < 2; b++) {
+    for (k = 1; k < conf.jobs; k++) {
+      if (jobs[b][k].running) {
+        pthread_join (threads[k], &status);
+        jobs[b][k].running = 0;
+      }
+
+      if (jobs[b][k].p) {
+        process_lines_thread (&jobs[b][k]);
+        cnt += jobs[b][k].cnt;
+        jobs[b][k].cnt = 0;
+        test &= jobs[b][k].test;
+        jobs[b][k].p = 0;
+      }
+    }
   }
 
-  /* if no data was available to read from (probably from a pipe) and
-   * still in test mode, we simply return until data becomes available */
-  if (!s && (errno == EAGAIN || errno == EWOULDBLOCK) && test)
+  free_jobs (jobs);
+
+  /* if no data was available to read from (probably from a pipe) and still in
+   * test mode and still below the test count, we simply return until data
+   * becomes available */
+  if (!s && (errno == EAGAIN || errno == EWOULDBLOCK) && test && cnt < conf.num_tests)
     return 0;
 
-  /* fails if
-     - we're still reading the log but the test flag was still set
-     - ret flag is not 0, read_line failed
-     - reached the end of file, test flag was still set and we processed lines */
-  return (s && test) || ret || (!s && test && glog->processed);
+  return test;
 }
-#endif
 
 /* Read the given log file and attempt to mmap a fixed number of bytes so we
  * can compare its content on future runs.
@@ -2092,6 +2235,7 @@ read_lines (FILE *fp, GLog *glog, int dry_run) {
 int
 set_initial_persisted_data (GLog *glog, FILE *fp, const char *fn) {
   size_t len;
+  time_t now = time (0);
 
   /* reset the snippet */
   memset (glog->snippet, 0, sizeof (glog->snippet));
@@ -2104,6 +2248,7 @@ set_initial_persisted_data (GLog *glog, FILE *fp, const char *fn) {
   if ((fread (glog->snippet, len, 1, fp)) != 1 && ferror (fp))
     FATAL ("Unable to fread the specified log file '%s'", fn);
   glog->snippetlen = len;
+  localtime_r (&now, &glog->start_time);
 
   fseek (fp, 0, SEEK_SET);
 
