@@ -32,7 +32,6 @@
 #include <unistd.h>
 #include <errno.h>
 #include <string.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 
 #include "persistence.h"
@@ -343,7 +342,7 @@ migrate_si32_to_ii32 (GSMetric metric, const char *path, int module) {
 }
 
 static char *
-migrate_unique_key (char *key) {
+migrate_unique_key (const char *key) {
   char *nkey = NULL, *token = NULL, *ptr = NULL;
   char agent_hex[64] = { 0 };
   uint32_t delims = 0;
