@@ -58,6 +58,7 @@ typedef enum LOGTYPE {
   AWSS3,
   CADDY,
   AWSALB,
+  TRAEFIKCLF,
 } GLogType;
 
 /* predefined log times */
@@ -89,6 +90,7 @@ typedef struct GPreConfLog_ {
   const char *awss3;
   const char *caddy;
   const char *awsalb;
+  const char *traefikclf;
 } GPreConfLog;
 
 /* *INDENT-OFF* */
@@ -103,7 +105,7 @@ typedef struct GConf_
   const char *ignore_ips[MAX_IGNORE_IPS];       /* array of ips to ignore */
   const char *ignore_panels[TOTAL_MODULES];     /* array of panels to ignore */
   const char *ignore_referers[MAX_IGNORE_REF];  /* referrers to ignore */
-  const char *ignore_status[MAX_IGNORE_STATUS]; /* status to ignore */
+  int ignore_status[MAX_IGNORE_STATUS]; /* status to ignore */
   const char *output_formats[MAX_OUTFORMATS];   /* output format, e.g. , HTML */
   const char *sort_panels[TOTAL_MODULES];       /* sorting options for each panel */
   const char *static_files[MAX_EXTENSIONS];     /* static extensions */
@@ -154,7 +156,8 @@ typedef struct GConf_
   int client_err_to_unique_count;   /* count 400s as visitors */
   int code444_as_404;               /* 444 as 404s? */
   int color_scheme;                 /* color scheme */
-  int crawlers_only ;               /* crawlers only */
+  int chunk_size;                   /* chunk size for each thread */
+  int crawlers_only;                /* crawlers only */
   int daemonize;                    /* run program as a Unix daemon */
   const char *username;             /* user to run program as */
   int double_decode;                /* need to double decode */
@@ -166,6 +169,7 @@ typedef struct GConf_
   int unknowns_as_crawlers;         /* unknown OS and browsers are classified as crawlers */
   int ignore_qstr;                  /* ignore query string */
   int ignore_statics;               /* ignore static files */
+  int jobs;                         /* multi-thread jobs count */
   int json_pretty_print;            /* pretty print JSON data */
   int list_agents;                  /* show list of agents per host */
   int load_conf_dlg;                /* load curses config dialog */
@@ -174,7 +178,7 @@ typedef struct GConf_
   int mouse_support;                /* add curses mouse support */
   int no_color;                     /* no terminal colors */
   int no_strict_status;             /* don't enforce 100-599 status codes */
-  int no_column_names;              /* don't show col names on termnal */
+  int no_column_names;              /* don't show col names on terminal */
   int no_csv_summary;               /* don't show overall metrics */
   int no_html_last_updated;         /* don't show HTML last updated field */
   int no_ip_validation;             /* don't validate client IP addresses */
