@@ -76,37 +76,34 @@ char *int2str (int d, int width);
 char *left_pad_str (const char *s, int indent);
 char *ltrim (char *s);
 char *regex_extract_string (const char *str, const char *regex, int max_groups, char const **err);
-char *replace_str (const char *str, const char *old, const char *new);
 char *rtrim (char *s);
-char *secs_to_str (int secs);
-char *strtoupper(char *str);
+char *strtoupper (char *str);
 char *substring (const char *str, int begin, int len);
 char *trim_str (char *str);
 char *u322str (uint32_t d, int width);
 char *u642str (uint64_t d, int width);
 char *unescape_str (const char *src);
 char *usecs_to_str (unsigned long long usec);
-const char *verify_status_code (char *str);
-const char *verify_status_code_type (const char *str);
+const char *verify_status_code (int code);
+const char *verify_status_code_type (int code);
 int convert_date (char *res, const char *data, const char *from, const char *to, int size);
 int count_matches (const char *s1, char c);
 int find_output_type (char **filename, const char *ext, int alloc);
-int hide_referer (const char *ref);
-int ignore_referer (const char *ref);
+int hide_referer (const char *host);
+int ignore_referer (const char *host);
 int intlen (uint64_t num);
-int invalid_ipaddr (char *str, int *ipvx);
+int invalid_ipaddr (const char *str, int *ipvx);
 int ip_in_range (const char *ip);
-int ptr2int(char *ptr);
+int ptr2int (char *ptr);
 int str2int (const char *date);
 int str_inarray (const char *s, const char *arr[], int size);
 int str_to_time (const char *str, const char *fmt, struct tm *tm, int tz);
 int valid_output_type (const char *filename);
 off_t file_size (const char *filename);
 size_t append_str (char **dest, const char *src);
-uint32_t djb2(const unsigned char *str);
-uint32_t ip_to_binary (const char *ip);
+uint32_t djb2 (const unsigned char *str);
 uint64_t u64encode (uint32_t x, uint32_t y);
-void genstr(char *dest, size_t len);
+void genstr (char *dest, size_t len);
 void set_tz (void);
 void strip_newlines (char *str);
 void u64decode (uint64_t n, uint32_t * x, uint32_t * y);
